from functools import lru_cache, wraps
from weakref import ref


def memoized_method(*lru_args, **lru_kwargs):
    'http://stackoverflow.com/a/33672499/1067833'
    def decorator(func):
        @wraps(func)
        def wrapped_func(self, *args, **kwargs):
            # We're storing the wrapped method inside the instance. If we had
            # a strong reference to self the instance would be never
            # garbage collected.
            self_weak = ref(self)

            @wraps(func)
            @lru_cache(*lru_args, **lru_kwargs)
            def cached_method(*args, **kwargs):
                return func(self_weak(), *args, **kwargs)
            setattr(self, func.__name__, cached_method)
            return cached_method(*args, **kwargs)
        return wrapped_func
    return decorator
