from functools import lru_cache
from re import compile as re_compile

from django.utils.encoding import force_str

_FIRST_CAP_RE = re_compile('(.)([A-Z][a-z]+)')
_ALL_CAP_RE = re_compile('([a-z0-9])([A-Z])')


@lru_cache(maxsize=30)
def dash_case_convert(name) -> str:
    """
    Convert a passed name in `CamelCase` to `dash-case`.

    Copied from http://stackoverflow.com/a/1176023/1067833

    CAUTION: this function doesn't deal with spaces!
    """
    s1 = _FIRST_CAP_RE.sub(r'\1-\2', force_str(name))
    return _ALL_CAP_RE.sub(r'\1-\2', s1).lower()
