from django.utils.functional import lazy
from django.utils.safestring import SafeString, _safety_decorator, mark_safe
from django.utils.translation import gettext_lazy as _gettext_lazy

from .types import StrOrPromise


def _mark_safe(s):
    """
    Keep this in sync with `django.utils.safestring.mark_safe`!
    This is separated because the `keep_lazy` decorator makes the
    original `mark_safe` unpickable for cache storage.
    So if you use the original, you'll get pickle errors from caching.
    """
    if hasattr(s, "__html__"):
        return s
    if callable(s):
        return _safety_decorator(mark_safe, s)
    return SafeString(s)


def gettext_safelazy(message: StrOrPromise):
    'Return a translated lazy `SafeString`.'
    unsafe = _gettext_lazy(message=message)
    return lazy(_mark_safe, SafeString)(s=unsafe)
